"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Util = void 0;
const Column_1 = require("../structures/Column");
const DatabaseError_1 = require("../structures/DatabaseError");
class Util extends null {
    constructor() { }
    static mergeDefault(target, source) {
        for (const [key, value] of Object.entries(source)) {
            const current = target[key];
            const def = source[key];
            if (current === undefined || current === null) {
                target[key] = def;
            }
            else if (!Array.isArray(def) && typeof def === "object") {
                Util.mergeDefault(current, def);
            }
        }
        return target;
    }
    static makeWhereOptions(options) {
        var _a, _b;
        if (!options)
            return "";
        const opts = ["WHERE"];
        if (options.where) {
            const ops = Array.isArray(options.where) ? options.where : [options.where];
            let i = 0;
            for (const where of ops) {
                if (where.endsWith !== undefined) {
                    opts.push(`${where.column} ${where.not ? `NOT` : ""} LIKE '%${where.endsWith}'`);
                }
                else if (where.startsWith !== undefined) {
                    opts.push(`${where.column} ${where.not ? `NOT` : ""} LIKE '${where.startsWith}%'`);
                }
                else if (where.lesser !== undefined) {
                    opts.push(`${where.column} ${where.not ? `NOT` : ""} < ${where.lesser}`);
                }
                else if (where.greater !== undefined) {
                    opts.push(`${where.column} ${where.not ? `NOT` : ""} > ${where.greater}`);
                }
                else if (where.equals !== undefined) {
                    opts.push(`${where.column} ${where.not ? `NOT` : ""} = ${typeof where.equals === "string" ? `'${where.equals}'` : where.equals}`);
                }
                else if (where.in !== undefined) {
                    opts.push(`${where.column} ${where.not ? `NOT` : ""} IN (${where.in.map(n => typeof n === "string" ? `'${n}'` : n).join(", ")})`);
                }
                else if (where.includes !== undefined) {
                    opts.push(`${where.column} ${where.not ? `NOT` : ""} LIKE '%${where.includes}%'`);
                }
                else if (where.lesserEquals !== undefined) {
                    opts.push(`${where.column} ${where.not ? `NOT` : ""} <= ${where.lesserEquals}`);
                }
                else if (where.greaterEquals !== undefined) {
                    opts.push(`${where.column} ${where.not ? `NOT` : ""} >= ${where.greaterEquals}`);
                }
                else if (where.notEqual !== undefined) {
                    opts.push(`${where.column} ${where.not ? `NOT` : ""} != ${typeof where.notEqual === "string" ? `'${where.notEqual}'` : where.notEqual}`);
                }
                i++;
                if (ops[i]) {
                    if (where.or) {
                        opts.push(`OR`);
                    }
                    else {
                        opts.push(`AND`);
                    }
                }
            }
        }
        if (options.sort) {
            const columns = Array.isArray(options.sort.column) ? options.sort.column : [options.sort.column];
            opts.push(`ORDER BY (${columns.join((_a = options.sort.operator) !== null && _a !== void 0 ? _a : "+")}) ${(_b = options.sort.order) !== null && _b !== void 0 ? _b : "DESC"}`);
        }
        if (options.limit) {
            opts.push(`LIMIT ${options.limit}`);
        }
        if (options.offset) {
            opts.push(`OFFSET ${options.offset}`);
        }
        return opts.join(" \n");
    }
    static makeColumnsFrom(data) {
        return Object.keys(data).join(", ");
    }
    static makeInsertValues(data, columns) {
        const got = [];
        for (const [key, val] of Object.entries(data)) {
            const column = columns.find(c => c.name === key);
            if (!column)
                throw new DatabaseError_1.DatabaseError(`Column ${key} does not exist.`);
            got.push(Column_1.Column.transformValue(val, column.type));
        }
        return got.join(", ");
    }
    static makeSetValues(data, columns) {
        const got = [];
        for (const [key, val] of Object.entries(data)) {
            const column = columns.find(c => c.name === key);
            if (!column)
                throw new DatabaseError_1.DatabaseError(`Column ${key} does not exist.`);
            got.push(`${key} = ${Column_1.Column.transformValue(val, column.type)}`);
        }
        return got.join(", \n");
    }
    static isNumber(n) {
        return isNaN(Number(n)) && n <= Number.MAX_SAFE_INTEGER && n >= Number.MIN_SAFE_INTEGER;
    }
}
exports.Util = Util;
//# sourceMappingURL=Util.js.map